@@ -11,7 +11,7 @@ namespace glz
    namespace detail
    {
       template <auto Opts>
-      inline void beve_to_json_number(auto&& tag, auto&& ctx, auto&& it, auto&& end, auto& out, auto&& ix) noexcept
+      inline void beve_to_json_number(auto&& tag, auto&& ctx, auto&& it, auto&& end, auto& out, auto& ix) noexcept
       {
          const auto number_type = (tag & 0b000'11'000) >> 3;
          const uint8_t byte_count = byte_count_lookup[tag >> 5];
@@ -21,7 +21,11 @@ namespace glz
                ctx.error = error_code::syntax_error;
                return;
             }
-            std::memcpy(&value, it, sizeof(T));
+            using V = std::remove_cvref_t<T>;
+            std::memcpy(&value, it, sizeof(V));
+            if constexpr (std::endian::native == std::endian::big) {
+               byteswap_le(value);
+            }
             to<JSON, T>::template op<Opts>(value, ctx, out, ix);
             it += sizeof(T);
          };
@@ -105,8 +109,15 @@ namespace glz
       }
 
       template <auto Opts, class Buffer>
-      inline void beve_to_json_value(auto&& ctx, auto&& it, auto&& end, Buffer& out, auto&& ix)
+      inline void beve_to_json_value(auto&& ctx, auto&& it, auto&& end, Buffer& out, auto&& ix,
+                                     uint32_t recursive_depth)
       {
+         // Check recursion depth limit
+         if (recursive_depth >= max_recursive_depth_limit) [[unlikely]] {
+            ctx.error = error_code::exceeded_max_recursive_depth;
+            return;
+         }
+
          if (it >= end) [[unlikely]] {
             ctx.error = error_code::syntax_error;
             return;
@@ -162,12 +173,6 @@ namespace glz
             else {
                ++ctx.indentation_level;
             }
-            // We use indentation_level for recursive depth limit even though this means we have a shorter limit with
-            // prettified output
-            if (ctx.indentation_level >= max_recursive_depth_limit) {
-               ctx.error = error_code::exceeded_max_recursive_depth;
-               return;
-            }
 
             const auto key_type = (tag & 0b000'11'000) >> 3;
             switch (key_type) {
@@ -196,8 +201,10 @@ namespace glz
                      dump<':'>(out, ix);
                   }
                   it += n;
-                  // convert the value
-                  beve_to_json_value<Opts>(ctx, it, end, out, ix);
+                  beve_to_json_value<Opts>(ctx, it, end, out, ix, recursive_depth + 1);
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
                   if (i != n_fields - 1) {
                      dump<','>(out, ix);
                      if constexpr (Opts.prettify) {
@@ -230,8 +237,10 @@ namespace glz
                   else {
                      dump<':'>(out, ix);
                   }
-                  // convert the value
-                  beve_to_json_value<Opts>(ctx, it, end, out, ix);
+                  beve_to_json_value<Opts>(ctx, it, end, out, ix, recursive_depth + 1);
+                  if (bool(ctx.error)) [[unlikely]] {
+                     return;
+                  }
                   if (i != n_fields - 1) {
                      dump<','>(out, ix);
                      if constexpr (Opts.prettify) {
@@ -274,7 +283,11 @@ namespace glz
                      ctx.error = error_code::unexpected_end;
                      return;
                   }
-                  std::memcpy(&value, it, sizeof(T));
+                  using V = std::remove_cvref_t<T>;
+                  std::memcpy(&value, it, sizeof(V));
+                  if constexpr (std::endian::native == std::endian::big) {
+                     byteswap_le(value);
+                  }
                   to<JSON, T>::template op<Opts>(value, ctx, out, ix);
                   it += sizeof(T);
                   if (i != n - 1) {
@@ -415,7 +428,7 @@ namespace glz
             }
             dump<'['>(out, ix);
             for (size_t i = 0; i < n; ++i) {
-               beve_to_json_value<Opts>(ctx, it, end, out, ix);
+               beve_to_json_value<Opts>(ctx, it, end, out, ix, recursive_depth + 1);
                if (bool(ctx.error)) [[unlikely]] {
                   return;
                }
@@ -441,52 +454,10 @@ namespace glz
                skip_compressed_int(ctx, it, end);
                if (bool(ctx.error)) return;
 
-               // IMPORTANT: Code was commented because variants should typically write out the JSON value directly
-               // This makes sense for auto-deducible values, which should be the default behavior.
-               // In the future we may want a compile time option that dumps the index for cases that cannot be
-               // auto-deduced
-               /*const auto index = int_from_compressed(ctx, it, end);
-
-               dump<'{'>(out, ix);
-               if constexpr (Opts.prettify) {
-                  ctx.indentation_level += Opts.indentation_width;
-                  dump<'\n'>(out, ix);
-                  dumpn<Opts.indentation_char>(ctx.indentation_level, out, ix);
-               }
-
-               if constexpr (Opts.prettify) {
-                  dump<R"("index": )">(out, ix);
-               }
-               else {
-                  dump<R"("index":)">(out, ix);
-               }
-
-               to<JSON, std::remove_cvref_t<decltype(index)>>::template op<Opts>(index, ctx, out, ix);
-
-               dump<','>(out, ix);
-               if constexpr (Opts.prettify) {
-                  dump<'\n'>(out, ix);
-                  dumpn<Opts.indentation_char>(ctx.indentation_level, out, ix);
-               }
-
-               if constexpr (Opts.prettify) {
-                  dump<R"("value": )">(out, ix);
-               }
-               else {
-                  dump<R"("value":)">(out, ix);
-               }*/
-
-               beve_to_json_value<Opts>(ctx, it, end, out, ix);
+               beve_to_json_value<Opts>(ctx, it, end, out, ix, recursive_depth + 1);
                if (bool(ctx.error)) [[unlikely]] {
                   return;
                }
-
-               /*if constexpr (Opts.prettify) {
-                  ctx.indentation_level -= Opts.indentation_width;
-                  dump<'\n'>(out, ix);
-                  dumpn<Opts.indentation_char>(ctx.indentation_level, out, ix);
-               }
-               dump<'}'>(out, ix);*/
                break;
             }
             case 2: {
@@ -508,10 +479,6 @@ namespace glz
                else {
                   ++ctx.indentation_level;
                }
-               if (ctx.indentation_level >= max_recursive_depth_limit) {
-                  ctx.error = error_code::exceeded_max_recursive_depth;
-                  return;
-               }
 
                if constexpr (Opts.prettify) {
                   dump<R"("layout": )">(out, ix);
@@ -536,7 +503,7 @@ namespace glz
                   dump<R"("extents":)">(out, ix);
                }
 
-               beve_to_json_value<Opts>(ctx, it, end, out, ix);
+               beve_to_json_value<Opts>(ctx, it, end, out, ix, recursive_depth + 1);
                if (bool(ctx.error)) [[unlikely]] {
                   return;
                }
@@ -554,7 +521,7 @@ namespace glz
                   dump<R"("value":)">(out, ix);
                }
 
-               beve_to_json_value<Opts>(ctx, it, end, out, ix);
+               beve_to_json_value<Opts>(ctx, it, end, out, ix, recursive_depth + 1);
                if (bool(ctx.error)) [[unlikely]] {
                   return;
                }
@@ -564,6 +531,9 @@ namespace glz
                   dump<'\n'>(out, ix);
                   dumpn<Opts.indentation_char>(ctx.indentation_level, out, ix);
                }
+               else {
+                  --ctx.indentation_level;
+               }
                dump<'}'>(out, ix);
                break;
             }
@@ -648,9 +618,9 @@ namespace glz
       context ctx{};
 
       while (it < end) {
-         detail::beve_to_json_value<Opts>(ctx, it, end, out, ix);
+         detail::beve_to_json_value<Opts>(ctx, it, end, out, ix, 0);
          if (bool(ctx.error)) {
-            return {ctx.error};
+            return {0, ctx.error};
          }
       }
 
