@@ -208,6 +208,28 @@ namespace glz
    // Trades O(1) hash lookup for O(N) linear search - faster for small structs (< ~8 fields)
    // due to cache effects, and much smaller binaries for embedded systems.
 
+   // ---
+   // size_t max_string_length = 0;
+   // Maximum length for string allocations when reading. 0 means no limit (default).
+   // When set, strings exceeding this length will fail with error_code::invalid_length.
+   // Useful for preventing memory exhaustion attacks from malicious input.
+   // Currently applies to BEVE format.
+
+   // ---
+   // size_t max_array_size = 0;
+   // Maximum size for array/vector allocations when reading. 0 means no limit (default).
+   // When set, arrays exceeding this size will fail with error_code::invalid_length.
+   // Useful for preventing memory exhaustion attacks from malicious input.
+   // Currently applies to BEVE format.
+
+   // ---
+   // bool allocate_raw_pointers = false;
+   // When true, allows Glaze to allocate memory for null raw pointers during deserialization using `new`.
+   // By default, Glaze refuses to read into null raw pointers because it would have to call `new`
+   // without any known way to delete the memory, making memory leaks easy.
+   // Enable this option only when you are prepared to manually manage the allocated memory.
+   // Works with JSON, BEVE, CBOR, and MSGPACK formats.
+
    struct append_arrays_opt_tag
    {};
 
@@ -437,6 +459,46 @@ namespace glz
       }
    }
 
+   consteval size_t check_max_string_length(auto&& Opts)
+   {
+      if constexpr (requires { Opts.max_string_length; }) {
+         return Opts.max_string_length;
+      }
+      else {
+         return 0; // 0 means no limit
+      }
+   }
+
+   consteval size_t check_max_array_size(auto&& Opts)
+   {
+      if constexpr (requires { Opts.max_array_size; }) {
+         return Opts.max_array_size;
+      }
+      else {
+         return 0; // 0 means no limit
+      }
+   }
+
+   consteval size_t check_max_map_size(auto&& Opts)
+   {
+      if constexpr (requires { Opts.max_map_size; }) {
+         return Opts.max_map_size;
+      }
+      else {
+         return 0; // 0 means no limit
+      }
+   }
+
+   consteval bool check_allocate_raw_pointers(auto&& Opts)
+   {
+      if constexpr (requires { Opts.allocate_raw_pointers; }) {
+         return Opts.allocate_raw_pointers;
+      }
+      else {
+         return false;
+      }
+   }
+
    consteval bool check_opening_handled(auto&& o) { return o.internal & uint32_t(opts_internal::opening_handled); }
 
    consteval bool check_closing_handled(auto&& o) { return o.internal & uint32_t(opts_internal::closing_handled); }
