@@ -434,7 +434,7 @@ namespace glz
       template <auto Opts = opts{}, class T>
          requires(Opts.format == JSON && (tuple_t<std::decay_t<T>> || is_std_tuple<std::decay_t<T>>))
       inline void handle_slice(const jmespath::ArrayParseResult& decomposed_key, T&& value, context& ctx, auto&& it,
-                               auto&& end)
+                               auto end)
       {
          if (skip_ws<Opts>(ctx, it, end)) {
             return;
@@ -531,7 +531,7 @@ namespace glz
       template <auto Opts = opts{}, class T>
          requires(Opts.format == JSON && readable_array_t<T>)
       inline void handle_slice(const jmespath::ArrayParseResult& decomposed_key, T&& value, context& ctx, auto&& it,
-                               auto&& end)
+                               auto end)
       {
          if (skip_ws<Opts>(ctx, it, end)) {
             return;
@@ -805,7 +805,7 @@ namespace glz
                      }
 
                      auto* start = it;
-                     skip_string_view<Opts>(ctx, it, end);
+                     skip_string_view(ctx, it, end);
                      if (bool(ctx.error)) [[unlikely]]
                         return;
                      const sv k = {start, size_t(it - start)};
@@ -899,7 +899,7 @@ namespace glz
                   }
 
                   auto* start = it;
-                  skip_string_view<Opts>(ctx, it, end);
+                  skip_string_view(ctx, it, end);
                   if (bool(ctx.error)) [[unlikely]]
                      return;
                   const sv k = {start, size_t(it - start)};
@@ -945,7 +945,7 @@ namespace glz
          buffer.resize(buffer.size() - padding_bytes);
       }
 
-      return {ctx.error, ctx.custom_error_message, size_t(it - start), ctx.includer_error};
+      return {size_t(it - start), ctx.error, ctx.custom_error_message};
    }
 
    // A "compiled" jmespath expression, which can be pre-computed for efficient traversal
@@ -978,7 +978,7 @@ namespace glz
    [[nodiscard]] inline error_ctx read_jmespath(const jmespath_expression& expression, T&& value, Buffer&& buffer)
    {
       if (bool(expression.error)) {
-         return {error_code::syntax_error, "JMESPath invalid expression"};
+         return {0, error_code::syntax_error, "JMESPath invalid expression"};
       }
 
       const auto& tokens = expression.tokens;
@@ -1102,7 +1102,7 @@ namespace glz
                         }
 
                         auto* start_pos = it;
-                        skip_string_view<Opts>(ctx, it, end);
+                        skip_string_view(ctx, it, end);
                         if (bool(ctx.error)) [[unlikely]]
                            return;
                         const sv k = {start_pos, size_t(it - start_pos)};
@@ -1196,7 +1196,7 @@ namespace glz
                      }
 
                      auto* start_pos = it;
-                     skip_string_view<Opts>(ctx, it, end);
+                     skip_string_view(ctx, it, end);
                      if (bool(ctx.error)) [[unlikely]]
                         return;
                      const sv k = {start_pos, size_t(it - start_pos)};
@@ -1252,7 +1252,7 @@ namespace glz
          buffer.resize(buffer.size() - padding_bytes);
       }
 
-      return {ctx.error, ctx.custom_error_message, size_t(it - start), ctx.includer_error};
+      return {size_t(it - start), ctx.error, ctx.custom_error_message};
    }
 
 }
